@@ -186,7 +186,7 @@ class Download:
         if self.cache and settings.read_cache:
             try:
                 html = self.cache[key]
-                if html and settings.pattern and not re.compile(settings.pattern, re.DOTALL | re.IGNORECASE).search(html):
+                if self.invalid_response(html, settings.pattern):
                     # invalid result from download
                     html = None
             except KeyError:
@@ -208,7 +208,10 @@ class Download:
         # attempt downloading content at URL
         while settings.num_retries >= 0 and html is None:
             settings.num_retries -= 1
-            self.proxy = self.get_proxy(settings.proxies)
+            if settings.proxy:
+                self.proxy = settings.proxy
+            else:
+                self.proxy = self.get_proxy(settings.proxies)
             # crawl slowly for each domain to reduce risk of being blocked
             self.throttle(url, delay=settings.delay, proxy=self.proxy) 
             html = self.fetch(url, headers=settings.headers, data=settings.data, proxy=self.proxy, user_agent=settings.user_agent, opener=settings.opener, pattern=settings.pattern)
@@ -344,6 +347,12 @@ class Download:
         return user_agent
 
 
+    def invalid_response(self, html, pattern):
+        """Return whether the response contains a regex error pattern 
+        """
+        return html is None or (pattern and not re.compile(pattern, re.DOTALL | re.IGNORECASE).search(html))
+
+
     def fetch(self, url, headers=None, data=None, proxy=None, user_agent=None, opener=None, pattern=None):
         """Simply download the url and return the content
         """
@@ -359,8 +368,8 @@ class Download:
                 opener.add_handler(urllib2.ProxyHandler({'http' : proxy}))
         
         headers = headers or {}
-        headers['User-agent'] = user_agent or self.get_user_agent(proxy)
-        headers['Accept-encoding'] = 'gzip'
+        headers['User-Agent'] = user_agent or self.get_user_agent(proxy)
+        headers['Accept-Encoding'] = 'gzip'
         for name, value in settings.default_headers.items():
             if name not in headers:
                 if name == 'Referer':
@@ -369,21 +378,22 @@ class Download:
         
         if isinstance(data, dict):
             # encode data for POST
-            data = urllib.urlencode(data) 
+            data = urllib.urlencode(sorted(data.items()))
         common.logger.info('Downloading %s %s' % (url, data or ''))
 
         try:
             request = urllib2.Request(url, data, headers)
             with contextlib.closing(opener.open(request)) as response:
                 content = response.read()
+                #print response.headers
                 if response.headers.get('content-encoding') == 'gzip':
                     # data came back gzip-compressed so decompress it          
                     content = gzip.GzipFile(fileobj=StringIO.StringIO(content)).read()
                 self.final_url = response.url # store where redirected to
-                if pattern and not re.compile(pattern, re.DOTALL | re.IGNORECASE).search(content):
+                if self.invalid_response(content, pattern):
                     # invalid result from download
                     content = None
-                    common.logger.warning('Content did not match expected pattern - %s' % url)
+                    common.logger.warning('Content did not match expected pattern: %s' % url)
                 self.response_code = str(response.code)
                 self.response_headers = dict(response.headers)
         except Exception, e:
@@ -442,65 +452,12 @@ class Download:
 
 
     def geocode(self, address, delay=5, read_cache=True, num_retries=1, language=None):
-        """Geocode address using Google's API and return dictionary of useful fields
+        gm = GoogleMaps(self)
+        return gm.geocode(address, delay, read_cache, num_retries, language)
 
-        address:
-            what to pass to geocode API
-        delay:
-            how long to delay between API requests
-        read_cache:
-            whether to load content from cache when exists
-        num_retries:
-            the number of times to try downloading
-        language:
-            the language to set
-        """
-        url = 'http://maps.google.com/maps/api/geocode/json?address=%s&sensor=false%s' % (urllib.quote_plus(address), '&language=' + language if language else '')
-        html = self.get(url, delay=delay, read_cache=read_cache, num_retries=num_retries)
-        results = collections.defaultdict(str)
-        if html:
-            try:
-                geo_data = json.loads(html)
-            except Exception, e:
-                common.logger.debug(str(e))
-                return {}
-            for result in geo_data.get('results', []):
-                for e in result['address_components']:
-                    types, value, abbrev = e['types'], e['long_name'], e['short_name']
-                    if 'street_number' in types:
-                        results['number'] = value
-                    elif 'route' in types:
-                        results['street'] = value
-                    elif 'postal_code' in types:
-                        results['postcode'] = value
-                    elif 'locality' in types:
-                        results['suburb'] = value
-                    elif 'administrative_area_level_1' in types:
-                        results['state'] = value
-                        results['state_code'] = abbrev
-                    elif 'administrative_area_level_2' in types:
-                        results['county'] = value
-                    elif 'administrative_area_level_3' in types:
-                        results['district'] = value
-                    elif 'country' in types:
-                        results['country'] = value
-                        results['country_code'] = value
-                results['full_address'] = result['formatted_address']
-                m = re.compile(r'"location" : {\s*"lat" : ([\d\-\.]+),\s*"lng" : ([\d\-\.]+)').search(html)
-                if m:
-                    results['lat'] = m.groups()[0].strip()
-                    results['lng'] = m.groups()[1].strip()
-                else:
-                    results['lat'] = result['geometry']['location']['lat']
-                    results['lng'] = result['geometry']['location']['lng']
-            if 'street' in results:
-                results['address'] = (results['number'] + ' ' + results['street']).strip()
-        if not results:
-            # error geocoding - try again later
-            common.logger.debug('Delete invalid geocode')
-            if self.cache:
-                self.cache[url] = ''
-        return results
+    def places(self, api_key, keyword, latitude, longitude, radius=10000, delay=5, num_retries=1, language='en'):
+        gm = GoogleMaps(self)
+        return gm.places(api_key, keyword, latitude, longitude, radius, delay, num_retries, language)
 
 
     def get_emails(self, website, max_depth=1, max_urls=10, max_emails=1):
@@ -622,31 +579,20 @@ class Download:
                 else:
                     raise KeyError()
             except KeyError:
-                # try online whois app
-                query_url = 'http://whois.chinaz.com/%s' % domain
-                html = self.get(query_url)
-                match = re.compile("<script src='(request.aspx\?domain=.*?)'></script>").search(html)
-                if match:
-                    script_url = urlparse.urljoin(query_url, match.groups()[0])
-                    text = self.get(script_url, read_cache=False)
-
-                if '@' not in text:
-                    if self.cache:
-                        del self.cache[query_url]
-                    # failed, so try local whois command
-                    r = subprocess.Popen(['whois', domain], stdout=subprocess.PIPE)
-                    start = time.time()
-                    while r.poll() is None:
-                        time.sleep(0.5)
-                        if time.time() - start > timeout:
-                            try:
-                                r.kill()
-                            except Exception, e:
-                                pass
-                            break
-                    if r.poll() != 1:
-                        text = r.communicate()[0]
-                
+                # try local whois command
+                r = subprocess.Popen(['whois', domain], stdout=subprocess.PIPE)
+                start = time.time()
+                while r.poll() is None:
+                    time.sleep(0.5)
+                    if time.time() - start > timeout:
+                        try:
+                            r.kill()
+                        except Exception, e:
+                            pass
+                        break
+                if r.poll() != 1:
+                    text = r.communicate()[0]
+            
                 if '@' in text:
                     if self.cache:
                         self.cache[key] = text
@@ -678,6 +624,149 @@ def get_redirect(url, html):
         return urlparse.urljoin(url, common.unescape(match.groups()[0].strip())) 
 
 
+class GoogleMaps:
+
+    def __init__(self, D):
+        self.D = D
+
+
+    def geocode(self, address, delay=5, read_cache=True, num_retries=1, language=None):
+        """Geocode address using Google's API and return dictionary of useful fields
+
+        address:
+            what to pass to geocode API
+        delay:
+            how long to delay between API requests
+        read_cache:
+            whether to load content from cache when exists
+        num_retries:
+            the number of times to try downloading
+        language:
+            the language to set
+        """
+        try:
+            address = address.encode('utf-8')
+        except UnicodeDecodeError:
+            common.logger.warning('Geocode failed to parse address and needed to cast to ascii: ' + address)
+            address = common.to_ascii(address)
+        geocode_url = 'http://maps.google.com/maps/api/geocode/json?address=%s&sensor=false%s' % (urllib.quote_plus(address), '&language=' + language if language else '')
+        geocode_html = self.D.get(geocode_url, delay=delay, read_cache=read_cache, num_retries=num_retries)
+        geocode_data = self.load_result(geocode_url, geocode_html)
+        for result in geocode_data.get('results', []):
+            return self.parse_location(result)
+        return collections.defaultdict(str)
+
+
+    def places(self, api_key, keyword, latitude, longitude, radius=10000, delay=5, num_retries=1, language='en'):
+        """Search the Google Place API for this keyword and location
+
+        api_key is the Google Places API key: https://developers.google.com/places/documentation/#Authentication
+        radius around the location can be a maximum 50000
+
+        Returns a list of up to 200 matching places
+        """
+        search_url_template = 'https://maps.googleapis.com/maps/api/place/radarsearch/json?key={0}&location={1},{2}&radius={3}&keyword={4}&sensor=false'
+        place_url_template = 'https://maps.googleapis.com/maps/api/place/details/json?key={0}&reference={1}&language={2}&sensor=false'
+
+        search_url = search_url_template.format(api_key, latitude, longitude, radius, keyword).replace(' ', '+')
+        search_html = self.D.get(search_url, delay=delay, num_retries=num_retries)
+        search_results = self.load_result(search_url, search_html)
+
+        place_results = []
+        # iterate search results
+        for search_result in search_results.get('results', []):
+            reference = search_result['reference']
+            # found a new place
+            place_url = place_url_template.format(api_key, reference, language)
+            place_html = self.D.get(place_url, delay=delay, num_retries=num_retries)
+
+            place = self.load_result(place_url, place_html)
+            if place:
+                place = place['result']
+                result = self.parse_location(place)
+                result['name'] = place['name']
+                result['categories'] = place['types']
+                result['phone'] = place.get('formatted_phone_number', '')
+                result['website'] = place.get('website', '')
+                place_results.append(result)
+        return place_results
+
+
+    def load_result(self, url, html):
+        """Parse the result from API
+
+        If JSON is well formed and status is OK then will return result
+        Else will return an empty dict
+        """
+        if html:
+            try:
+                search_data = json.loads(html)
+            except ValueError as e:
+                common.logger.debug(str(e))
+            else:
+                status = search_data['status']
+                if status == 'OK':
+                    return search_data
+                elif status == 'ZERO_RESULTS':
+                    pass
+                elif status == 'OVER_QUERY_LIMIT':
+                    # error geocoding - try again later
+                    self.D.cache[url] = ''
+                elif status in ('REQUEST_DENIED', 'INVALID_REQUEST'):
+                    common.logger.info('{0}: {1}'.format(status, url))
+        return {}
+
+
+    def parse_location(self, result):
+        """Parse address data from Google's geocoding response into a more usable flat structure
+        
+        Example: https://developers.google.com/maps/documentation/geocoding/#JSON
+        """
+        results = collections.defaultdict(str)
+        for e in result['address_components']:
+            # parse address compenents into flat layer
+            types, value, abbrev = e['types'], e['long_name'], e['short_name']
+            if 'street_number' in types:
+                results['number'] = value
+            elif 'route' in types:
+                results['street'] = value
+            elif 'postal_code' in types:
+                results['postcode'] = value
+            elif 'locality' in types:
+                results['suburb'] = value
+            elif 'administrative_area_level_1' in types:
+                results['state'] = value
+                results['state_code'] = abbrev
+            elif 'administrative_area_level_2' in types:
+                results['county'] = value
+            elif 'administrative_area_level_3' in types:
+                results['district'] = value
+            elif 'country' in types:
+                results['country'] = value
+                results['country_code'] = abbrev
+       
+        # extract addresses 
+        results['full_address'] = result['formatted_address']
+        if 'street' in results:
+            results['address'] = (results['number'] + ' ' + results['street']).strip()
+
+        # extract latitude longitude
+        #m = re.compile(r'"location" : {\s*"lat" : ([\d\-\.]+),\s*"lng" : ([\d\-\.]+)').search(html)
+        #if m:
+        #    results['lat'] = m.groups()[0].strip()
+        #    results['lng'] = m.groups()[1].strip()
+        #else:
+        results['lat'] = result['geometry']['location']['lat']
+        results['lng'] = result['geometry']['location']['lng']
+        return results
+
+
+
+class StopCrawl(Exception):
+    """Raise this exception to interrupt crawl
+    """
+    pass
+
 
 def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=None, wait_finish=True, reuse_queue=False, max_queue=1000, **kwargs):
     """Download these urls in parallel
@@ -715,6 +804,7 @@ def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=No
         """
         processing = collections.deque() # to track whether are still downloading
         discovered = {} # the URL's that have been discovered
+        running = True
 
         def __init__(self):
             threading.Thread.__init__(self)
@@ -723,7 +813,7 @@ def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=No
             D = Download(**kwargs)
             queue = pdict.Queue(settings.queue_file)
 
-            while seed_urls or DownloadThread.processing:
+            while DownloadThread.running and (seed_urls or DownloadThread.processing):
                 # keep track that are processing url
                 DownloadThread.processing.append(1) 
                 try:
@@ -744,7 +834,11 @@ def threaded_get(url=None, urls=None, num_threads=10, dl=None, cb=None, depth=No
                                 # use callback to process downloaded HTML
                                 result = cb(D, url, html)
 
-                            except Exception, e:
+                            except StopCrawl:
+                                common.logger.info('Stopping crawl signal')
+                                DownloadThread.running = False
+
+                            except Exception:
                                 # catch any callback error to avoid losing thread
                                 common.logger.exception('\nIn callback for: ' + str(url))
 
@@ -974,12 +1068,17 @@ class CrawlerCallback:
             # extract links to continue crawling
             links_re = re.compile('<a[^>]+href=["\'](.*?)["\']', re.IGNORECASE)
             for link in links_re.findall(html):
-                link = normalize(link)
-                if link not in CrawlerCallback.found:
-                    CrawlerCallback.found[link] = depth + 1
-                    if valid(link):
-                        # is a new link
-                        outstanding.append(link)
-                        if len(outstanding) == self.max_links:
-                            break
+                try:
+                    link = normalize(link)
+                except UnicodeDecodeError as e:
+                    # unicode error when joining url
+                    common.logger.info(e)
+                else:
+                    if link not in CrawlerCallback.found:
+                        CrawlerCallback.found[link] = depth + 1
+                        if valid(link):
+                            # is a new link
+                            outstanding.append(link)
+                            if len(outstanding) == self.max_links:
+                                break
         return outstanding
